#!/bin/sh

release=`uname --release`
instdir=`pwd`
opsystem=`uname -s`
gnomedir="/usr/share/gnome/apps"
cpoptions="-a -f"
if [ "$opsystem" = "FreeBSD" ]
then
	gnomedir="/usr/X11R6/share/gnome/apps"
	cpoptions="-p -R -f"
fi

if [ "$opsystem" = "Linux" ]
then
echo "You are running currently Linux $release"
echo "This driver has only been tested on Linux"
echo "2.4.7-10, 2.4.18-3, 2.4.18-14 and 2.4.20-8,"
echo "if you wish to continue please type Y and enter now."
read answer
if [ "$answer" != "Y" ]
then
	echo "Installation aborted."
	exit 1
fi                   	

if [ "$release" = "2.4.7-10" ]
then

# Red Hat 7.2
# 2.4.7-10    i386   804083
# 2.4.7-10    i686   802068
# 2.4.7-10    athlon 812901
# 2.4.7-10smp i586   854281
# 2.4.7-10smp i686   858769
# 2.4.7-10smp athlon 855807

bootsize=`(ls -l /boot/vmlinuz-2.4.7-10 | awk '{ print $5 }') 2>/dev/null`

case $bootsize in
	804083)
		kern="i386"
		modname="fx-i386.o"
	;;
	802068)
		kern="i686"
		modname="fx.o"
	;;
	812901)
		kern="athlon"
		modname="fx-athlon.o"
	;;
	*)
		kern="none"
	;;
esac

bootsize=`(ls -l /boot/vmlinuz-2.4.7-10smp | awk '{ print $5 }') 2>/dev/null`

case $bootsize in
	854281)
		smpkern="i586"
		smpmodname="fx-i586.o"
	;;
	858769)
		smpkern="i686"
		smpmodname="fx.o"
	;;
	855807)
		smpkern="athlon"
		smpmodname="fx-athlon.o"
	;;
	*)
		smpkern="none"
	;;
esac
success="yes"
fi

if [ "$release" = "2.4.18-3" ]
then
        
# Red Hat 7.3
# 2.4.18-3    i386   1041347
# 2.4.18-3    i686   1030147
# 2.4.18-3    athlon 1052336
# 2.4.18-3smp i586   1105332
# 2.4.18-3smp i686   1108097
# 2.4.18-3smp athlon 1103872

bootsize=`(ls -l /boot/vmlinuz-2.4.18-3 | awk '{ print $5 }') 2>/dev/null`

case $bootsize in
	1041347)
		kern="i386"
		modname="fx-i386.o"
	;;
	1030147)
		kern="i686"
		modname="fx.o"
	;;
	1052336)
		kern="athlon"
		modname="fx-athlon.o"
	;;
	*)
		kern="none"
	;;
esac

bootsize=`(ls -l /boot/vmlinuz-2.4.18-3smp | awk '{ print $5 }') 2>/dev/null`

case $bootsize in
	1105332)
		smpkern="i586"
		smpmodname="fx-i586.o"
	;;
	1108097)
		smpkern="i686"
		smpmodname="fx.o"
	;;
	1103872)
		smpkern="athlon"
		smpmodname="fx-athlon.o"
	;;
	*)
		smpkern="none"
	;;
esac
success="yes"
fi

if [ "$release" = "2.4.18-14" ]
then
        
# Red Hat 8.0
# 2.4.18-14    i586   1085191
# 2.4.18-14    i686   1125194
# 2.4.18-14    athlon 1114125
# 2.4.18-14smp i686   1206433
# 2.4.18-14smp athlon 1188204

bootsize=`(ls -l /boot/vmlinuz-2.4.18-14 | awk '{ print $5 }') 2>/dev/null`

case $bootsize in
	1085191)
		kern="i586"
		modname="fx-i586.o"
	;;
	1125194)
		kern="i686"
		modname="fx.o"
	;;
	1114125)
		kern="athlon"
		modname="fx-athlon.o"
	;;
	*)
		kern="none"
	;;
esac

bootsize=`(ls -l /boot/vmlinuz-2.4.18-14smp | awk '{ print $5 }') 2>/dev/null`

case $bootsize in
	1206433)
		smpkern="i686"
		smpmodname="fx.o"
	;;
	1188204)
		smpkern="athlon"
		smpmodname="fx-athlon.o"
	;;
	*)
		smpkern="none"
	;;
esac
success="yes"
fi

if [ "$release" = "2.4.20-8" ]
then
        
# Red Hat 9.0
# 2.4.20-8    i586   1085191
# 2.4.20-8    i686   1125194
# 2.4.20-8    athlon 1114125
# 2.4.20-8smp i686   1206433
# 2.4.20-8smp athlon ?

bootsize=`(ls -l /boot/vmlinuz-2.4.20-8 | awk '{ print $5 }') 2>/dev/null`

case $bootsize in
	1096436)
		kern="i586"
		modname="fx-i586.o"
	;;
	1122186)
		kern="i686"
		modname="fx.o"
	;;
	1117254)
		kern="athlon"
		modname="fx-athlon.o"
	;;
	*)
		kern="none"
	;;
esac

bootsize=`(ls -l /boot/vmlinuz-2.4.20-8smp | awk '{ print $5 }') 2>/dev/null`

case $bootsize in
	1211291)
		smpkern="i686"
		smpmodname="fx.o"
	;;
	1200475)
		smpkern="athlon"
		smpmodname="fx-athlon.o"
	;;
	*)
		smpkern="none"
	;;
esac
success="yes"
fi

if [ "$success" = "yes" ]
then

echo "Detected $release kernel    : $kern"
echo "Detected ${release}smp kernel : $smpkern"
echo "If this is correct type Y and enter now."
read answer
if [ "$answer" != "Y" ]
then
	echo "$release kernel:"
	echo "        1: i386"
	echo "        2: i586"
	echo "        3: i686"
	echo "        4: Athlon"
	read answer
	case $answer in
		1)
			kern="i386"
			modname="fx-i386.o"
		;;
		2)
			kern="i586"
			modname="fx-i586.o"
		;;
		3)
			kern="i686"
			modname="fx.o"
		;;
		4)
			kern="athlon"
			modname="fx-athlon.o"
		;;
		*)
			kern="none"
			modname=
		;;
	esac
	echo "${release}smp kernel:"
	echo "        1: i586"
	echo "        2: i686"
	echo "        3: Athlon"
	echo
	read answer
	case $answer in
		1)
			smpkern="i586"
			smpmodname="fx-i586.o"
		;;
		2)
			smpkern="i686"
			smpmodname="fx.o"
		;;
		3)
			smpkern="athlon"
			smpmodname="fx-athlon.o"
		;;
		*)
			smpkern="none"
			smpmodname=
		;;
	esac
fi

echo "Installing for $release kernel    : $kern"
echo "Installing for ${release}smp kernel : $smpkern"

if [ -d sys ]
then
	cd sys
	for file in `ls -d 2.4.*`
	do
		if [ -d /lib/modules/"$file" ]
		then
			fxfile=`echo $file | grep smp`
			modfile="$modname"
			testrelease="$release"
			if [ "$fxfile" = "$file" ]
			then
				modfile="$smpmodname"
				testrelease="${release}smp"
			fi
			if [ "$file" = "$testrelease" ]
			then
				if [ -f "$file"/"$modfile" ]
				then
					echo "Installing /lib/modules/${file}/kernel/drivers/net/fx.o:"
					mkdir -p /lib/modules/"$file"/kernel/drivers/net/
					cp -a "$file"/"$modfile" /lib/modules/"$file"/kernel/drivers/net/fx.o
				fi
			fi
		fi
	done
	cd ..
fi
fi

if [ "$success" != "yes" ]
then
	echo "An unsupported kernel has been detected.  Please use the"
	echo "driver source package from the source directory and build"
	echo "a driver for your kernel before proceeding!"
	echo "If you still wish to proceed please type Y and enter now."
	read answer
	if [ "$answer" != "Y" ]
	then
		echo "Installation aborted."
		exit 1
	fi
fi

for file in `ls -d /lib/modules/2.4.*`
do
    if [ -f "$file"/kernel/drivers/net/fx.o ]
    then
	driver_copied="ok"
    fi
done
if [ "$driver_copied" = "ok" ]
then
    echo "Driver was installed successfuly"
else
    echo "Driver was not installed"
    echo "Please run Install.sh again or use the driver source"
    echo "to build a new driver"
    exit 1
fi

echo "Making changes to system files, please wait..."

# Create a new /etc/fx.conf with the currently loaded
# network drivers.
modlist=`/sbin/lsmod | cut --delimiter=' ' --fields=1`

cd /lib/modules/${release}/kernel/drivers/net/

if [ -f /etc/fx.conf ]
then
	mv -f /etc/fx.conf /etc/fx.conf.bak
fi

for file in `ls *.o`
do
	shortname=`echo ${file} | cut --delimiter='.' --fields=1`
	overloaded=`echo ${shortname} | grep _fx`
	if [ "$overloaded" != "$shortname" ]
	then
		for mod in ${modlist}
		do
			if [ "$mod" != "fx" ]
			then
				if [ "$mod" == "$shortname" ]
				then
					echo "${shortname} " >> /etc/fx.conf
				fi
				if [ "$mod" == "${shortname}_fx" ]
				then
					echo "${shortname} " >> /etc/fx.conf
				fi
			fi
		done
	fi
done

cd ${instdir}

# Replace occurances of eth# with fx# in the system configuration.
for file in `find /etc`
do
	if [ -f "$file" ]
	then
		cp -p "$file" "$file".tmp
		sed 's/eth\([0-9]\)/fx\1/g' $file > "$file".tmp
		mv "$file".tmp "$file"
	fi
done

for i in 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19
do
	if [ -r "/etc/sysconfig/network-scripts/ifcfg-eth$i" ]
	then
		cp -f -a "/etc/sysconfig/network-scripts/ifcfg-eth$i" "/etc/sysconfig/network-scripts/ifcfg-fx$i"
	fi
	if [ -r "/etc/sysconfig/networking/devices/ifcfg-eth$i" ]
	then
		cp -f -a "/etc/sysconfig/networking/devices/ifcfg-eth$i" "/etc/sysconfig/networking/devices/ifcfg-fx$i"
	fi
	if [ -r "/etc/sysconfig/networking/profiles/default/ifcfg-eth$i" ]
	then
		cp -f -a "/etc/sysconfig/networking/profiles/default/ifcfg-eth$i" "/etc/sysconfig/networking/profiles/default/ifcfg-fx$i"
	fi
done


echo "When the InJoy Firewall application is NOT running, the driver should:"
echo "1: Allow all IP traffic (default)"
echo "2: Block all IP traffic"
read answer

for file in `ls -d /lib/modules/2.4.*`
do
    if [ -f "$file"/kernel/drivers/net/fx.o ]
    then
	if [ "$answer" = "2" ]
	then
	    mv -f "$file"/kernel/drivers/net/fx.o "$file"/kernel/drivers/net/fx.bak
	    cat "$file"/kernel/drivers/net/fx.bak | sed s@FXENABLED1@FXENABLED0@ > "$file"/kernel/drivers/net/fx.o
	else
	    mv -f "$file"/kernel/drivers/net/fx.o "$file"/kernel/drivers/net/fx.bak
	    cat "$file"/kernel/drivers/net/fx.bak | sed s@FXENABLED0@FXENABLED1@ > "$file"/kernel/drivers/net/fx.o
	fi
	rm -f "$file"/kernel/drivers/net/fx.bak
    fi
done

cp -a -f sys/S10fx /etc/rc.d/rc3.d/
cp -a -f sys/S90gateway /etc/rc.d/rc3.d/
cp -a -f sys/S10fx /etc/rc.d/rc5.d/
cp -a -f sys/S90gateway /etc/rc.d/rc5.d/
cp -a -f sys/fx /etc/rc.d/init.d/
cp -a -f sys/fx_objcopy /bin/
cp -a -f sys/findsym /bin/
fi

# Handle FreeBSD specific install
if [ "$opsystem" = "FreeBSD" ]
then
	cp ${cpoptions} sys/gateway.sh /usr/local/etc/rc.d/
	if [ ! -e /etc/fx.conf ]
	then
        	# Create an /etc/fx.conf from the interface list
		for iface in `/sbin/ifconfig -l`
		do
        		found="0"
                        # Eliminate interfaces that we shouldn't firewall
			for num in 0 1 2 3 4 5 6 7 8 9
        	        do
                		if [ "$iface" = "faith${num}" ]
	                        then
        	                	found="1"
                	        fi
	                	if [ "$iface" = "lo${num}" ]
        	                then
                	        	found="1"
	                        fi
        		        	if [ "$iface" = "ppp${num}" ]
                        	then
	                        	found="1"
        	                fi
                		if [ "$iface" = "sl${num}" ]
	                        then
        	                	found="1"
                	        fi
                		if [ "$iface" = "tun${num}" ]
	                        then
        	                	found="1"
                	        fi
                		if [ "$iface" = "lp${num}" ]
	                        then
        	                	found="1"
                	        fi
                		if [ "$iface" = "ng${num}" ]
	                        then
        	                	found="1"
                	        fi
	                done
                        # If our iterface is ok add it to fx.conf
                        if [ "$found" = "0" ]
                        then
                        	echo "$iface" >> /etc/fx.conf
                        fi
		done        
        fi
fi

if [ ! -e /etc/fx.conf ]
then
	touch /etc/fx.conf
fi
./Folder.sh
cp ${cpoptions} lib/* /usr/lib
/sbin/ldconfig

res="0"
if [ -e /etc/fx_forward ]
then
	res=`(cat /etc/fx_forward) 2>/dev/null`
fi

if [ "$res" = "0" ]
then
	rm -f /etc/fx_forward
	echo "Gateway features such as Internet Sharing require IP forwarding to be enabled."
	echo "1: Enable IP Forwarding"
	echo "2: Leave setting unmodified (default)"
	read answer
	if [ "$answer" = "1" ]
	then
		echo 1 > /etc/fx_forward
	fi                   	
fi
rm -f /etc/fx_start
echo "Firewall Server startup mode:"
echo "1: Automatic"
echo "2: Manual"
read answer
if [ "$answer" = "1" ]
then
	echo "${instdir}/gateway" > /etc/fx_start
fi
echo "Installation complete, please reboot for changes to take effect."
