#!/bin/sh

release=`uname --release`
instdir=`pwd`
opsystem=`uname -s`
gnomedir="/usr/share/gnome/apps"
if [ "$opsystem" = "FreeBSD" ]
then
	gnomedir="/usr/X11R6/share/gnome/apps"
fi

echo "If you are sure you wish to continue with uninstallation please type Y now."
read answer
if [ "$answer" != "Y" ]
then
	echo "Uninstallation aborted."
	exit 1
fi                   	

if [ "$opsystem" = "Linux" ]
then
echo Making changes to system files, please wait...

# Replace occurances of eth# with fx# in the system configuration.
for file in `find /etc`
do
	if [ -f "$file" ]
	then
		cp -p "$file" "$file".tmp
		sed 's/fx\([0-9]\)/eth\1/g' $file > "$file".tmp
		mv "$file".tmp "$file"
	fi
done

for file in `ls -d /lib/modules/2.4.*`
do
	if [ -r "$file"/kernel/drivers/net/fx.o ]
        then
		echo "Removing ${file}/kernel/drivers/net/fx.o:"
        	rm -f "$file"/kernel/drivers/net/fx.o
        fi

	for rmodfile in `cat /etc/fx.conf`
	do
		if [ -r "$file"/kernel/drivers/net/"$rmodfile"_fx.o ]
		then
		        echo "Removing ${file}/kernel/drivers/net/${rmodfile}_fx.o:"
			rm -f "$file"/kernel/drivers/net/"$rmodfile"_fx.o
		fi
	done
done

rm -f /etc/rc.d/rc3.d/S10fx
rm -f /etc/rc.d/rc3.d/S90gateway
rm -f /etc/rc.d/rc5.d/S10fx
rm -f /etc/rc.d/rc5.d/S90gateway
rm -f /etc/rc.d/init.d/fx
rm -f /bin/fx_objcopy
rm -f /bin/findsym
rm -f /etc/sysconfig/network-scripts/ifcfg-fx*
rm -f /etc/sysconfig/networking/devices/ifcfg-fx*
rm -f /etc/sysconfig/networking/profiles/default/ifcfg-fx*
fi

rm -f /etc/fx.conf
rm -f /etc/fx_forward
rm -f /etc/fx_start

./menupars -c firewall.xml -d 2>/dev/null

if [ $? -ne 0 ]
then
    rm -rf "${gnomedir}/Internet/InJoy Firewall/"
    rm -rf "${HOME}/.gnome-desktop/InJoy Firewall/"
fi
rm -rf /dev/fx

rm -f /usr/lib/libfirewall.so*
rm -f /usr/lib/libfxip.so*
rm -f /usr/lib/libfxmasq.so*
rm -f /usr/lib/libfxutil.so*
rm -f /usr/lib/libipfrag.so*
rm -f /usr/lib/libfxipsec.so*
rm -f /usr/lib/libpptp.so*
rm -f /usr/lib/libpppoe.so*
rm -f /usr/lib/libdhcpd.so*
rm -f /usr/lib/libgenerate.so*
rm -f /usr/lib/libdw.so*
rm -f /usr/lib/libdwcompat.so*
rm -f /usr/share/pixmaps/gateway.png
rm -f /usr/share/pixmaps/fgui.png
rm -f /usr/share/pixmaps/logview.png
rm -f /usr/share/pixmaps/sync.png
rm -f /usr/share/pixmaps/uninstal.png

if [ "$opsystem" = "Linux" ]
then
	echo "Uninstallation complete, please reboot for changes to take effect."
fi
if [ "$opsystem" != "Linux" ]
then
	echo "Uninstallation complete."
fi
