/* fx.c: FX network filter driver
 *
 */

#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/sched.h>
#include <linux/types.h>
#include <linux/fcntl.h>
#include <linux/interrupt.h>
#include <linux/ptrace.h>
#include <linux/ioport.h>
#include <linux/in.h>
#include <linux/slab.h>
#include <linux/string.h>
#include <linux/init.h>
#include <linux/timer.h>
#include <asm/system.h>
#include <asm/bitops.h>
#include <asm/io.h>
#include <asm/dma.h>
#include <asm/system.h>
#include <asm/uaccess.h>
#include <linux/errno.h>

#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/skbuff.h>
#include <linux/devfs_fs_kernel.h>
#include <linux/if_arp.h>
#include <linux/module.h>
#include <linux/sched.h>
#include <linux/skbuff.h>
#include <linux/poll.h>
#include <linux/proc_fs.h>

/* devfs_register_chrdev calls register_chrdev and if you pass 0
 * as the major device number it dynamically allocates one.
 */

/* We are going to use the major number of the BDM commercial debugger
 * until we can obtain our own major number or devfs becomes the standard.
 */
#define FX_MAJOR 245

static int fx_major = -1;
static devfs_handle_t devfs_handle = NULL;
static struct proc_dir_entry *fx_ent = NULL;
static int ppp_instance = -1;

static int fx_xmit(struct sk_buff *skb, struct net_device *dev);
static struct net_device_stats *fx_get_stats(struct net_device *dev);
int __init fx_init(struct net_device *dev);
void dev_mon(unsigned long instance);

#define FX_MAX_INSTANCE 20

typedef struct _fx_buf {
	struct net_device *dev;
	struct net_device_stats stats;
	struct timer_list timer;
	int count, ppp;
	int (*xmit)(struct sk_buff *, struct net_device *);
	int (*init)(struct net_device *);
	struct sk_buff_head inbuf;
	wait_queue_head_t inwait;
	devfs_handle_t inhandle;
	struct sk_buff *skb_frag;
	int currentfrag;
	struct sk_buff_head outbuf;
	wait_queue_head_t outwait;
	devfs_handle_t outhandle;
	int fx_active;
} FxBuf;

FxBuf *fxroot[FX_MAX_INSTANCE];
int pleaseexit[FX_MAX_INSTANCE];
char EnablementCode[] = "FXENABLED1";

/* An always useful macro */
#define MIN(a, b) (((a) < (b)) ? (a) : (b))

/* Predeclare this function for fx_init */
int fx_startup(struct net_device *, int, int);


int _find_instance(struct net_device *blah)
{
	int z;

	for(z=0;z<FX_MAX_INSTANCE;z++)
		if(fxroot[z] && fxroot[z]->dev == blah)
			return z;
	return -1;
}

#pragma pack(1)
struct udphdr {
	unsigned short uh_sport;               /* source port */
	unsigned short uh_dport;               /* destination port */
	short          uh_ulen;                /* udp length */
	unsigned short uh_sum;                 /* udp checksum */
};

struct ip
{
	unsigned char  ip_hl:4;               /* header length */
	unsigned char  ip_v:4;                /* version */
	unsigned char  ip_tos;                /* type of service */
	unsigned short ip_len;                /* total length */
	unsigned short ip_id;                 /* identification */
	unsigned short ip_off;                /* fragment offset field */
	unsigned char  ip_ttl;                /* time to live */
	unsigned char  ip_p;                  /* protocol */
	unsigned short ip_sum;                /* checksum */
	unsigned long  ip_src, ip_dst;        /* source and dest address */
};
#pragma pack()

int IsDHCP(unsigned char *buffer)
{
	/* IP Packet */
	if(buffer[12] == 0x08 && buffer[13] == 0x00)
	{
		struct ip *pip = (struct ip *)&buffer[14];

		if(pip->ip_p == 17)
		{
			struct udphdr *pudp = (struct udphdr *)&buffer[14 + (pip->ip_hl * 4)];
			unsigned short dp = ntohs(pudp->uh_dport);
			unsigned short sp = ntohs(pudp->uh_sport);

			if(dp == 67 || sp == 67 || dp == 68 || sp == 68)
				return 1;
		}
	}
	return 0;
}

int is_dhcp(struct sk_buff *skb)
{
	static unsigned char buf[114];

	if(skb->mac.ethernet)
	{
		int len = MIN(skb->len + 14, 100);

		memcpy(buf, skb->mac.ethernet, 14);
		memcpy(&buf[14], skb->data, len);
		return IsDHCP(buf);
	}
	return 0;
}

/* Routines to deal with /proc */
static int fx_proc_read(char *buffer, char **start, off_t offset,
			   int length, int *eof, void *data)
{
	*start=buffer;
	
	/* Signal that this is the end
	 * of the file */
	if(offset > 0)
		return 0;

	if(length > 0)
	{
		int bytes = MIN(length, FX_MAX_INSTANCE);

		if(ppp_instance > -1)
		{
			buffer[0] = (char)ppp_instance;
			ppp_instance = -1;
		}
		else
		{
			int z;

			for(z=0;z<bytes;z++)
			{
				if(fxroot[z] && fxroot[z]->fx_active)
					buffer[z] = '1';
				else
					buffer[z] = '0';
			}
		}

		return bytes;
	}

	return 0;
}

static int fx_proc_write(struct file *file, const char *buffer, 
			 unsigned long count, void *data)
{
	if(count)
	{
		if(count == 3 && strncmp("PPP", buffer, 3)==0)
		{

			struct net_device *dev = kmalloc(sizeof(struct net_device), GFP_KERNEL);

			if (dev == NULL)
				return -ENOMEM;

			memset(dev, 0, sizeof(struct net_device));

			/* Initialize the device structure. */
			dev->hard_start_xmit = fx_xmit;

			/* Fill in the fields of the device structure with ethernet-generic values. */
			ether_setup(dev);

			dev->type = ARPHRD_PPP;
			dev->flags = IFF_POINTOPOINT | IFF_NOARP | IFF_MULTICAST;

			/* Create the PPP interface */
			fx_startup(dev, 0, 1);
		}
		else if(count == 4 && strncmp("PPP", buffer, 3) == 0)
		{
			int thisinstance = (int)buffer[3];

			if(thisinstance < FX_MAX_INSTANCE &&
			   fxroot[thisinstance] &&
			   fxroot[thisinstance]->dev &&
			   fxroot[thisinstance]->ppp)
			{
				unregister_netdev(fxroot[thisinstance]->dev);
				devfs_unregister(fxroot[thisinstance]->outhandle);
				skb_queue_purge(&fxroot[thisinstance]->outbuf);
				/* fxin# is only used for ethernet interfaces */
				kfree(fxroot[thisinstance]);
				fxroot[thisinstance] = NULL;
			}
		}
		else if(count == 2)
		{
			int index = buffer[0] - '0';

			if(index > -1 && index < FX_MAX_INSTANCE)
			{
				if(buffer[1] == '0' && fxroot[index])
					fxroot[index]->fx_active = 0;
				else if(buffer[1] == '1' && fxroot[index])
					fxroot[index]->fx_active = 1;
			}
		}
	}
	return count;
}


/* We'll make this simple and only expand two decimals */
int atoi(char *blah)
{
	int val = 0;

	if(blah && *blah)
		val = *blah - '0';
	blah++;
	if(blah && *blah)
	{
		val = val * 10;
		val += *blah - '0';
	}
	return val;
}

int _find_file_instance(struct file *file)
{
	int z, len = 0;
	char *name;

	if(!file || !file->f_dentry)
		return -1;

	name = (char *)file->f_dentry->d_name.name;
	len = file->f_dentry->d_name.len;

	for(z=0;z<len;z++)
	{
		if(name[z] >= '0' && name[z] <= '9')
		{
			int ret = atoi(&name[z]);
			if(ret > -1 && ret < FX_MAX_INSTANCE)
				return ret;
			else
				return -1;
		}
	}
	return -1;
}

/* Dealing with the in (from ethernet) device */
static unsigned int fx_in_dev_poll(struct file *file, poll_table * wait)
{
	int instance = _find_file_instance(file);

	if(instance > -1)
	{
		if(skb_queue_len(&(fxroot[instance]->inbuf)))
			return POLLIN | POLLRDNORM;

		poll_wait(file, &(fxroot[instance]->inwait), wait);

		if(skb_queue_len(&(fxroot[instance]->inbuf)))
			return POLLIN | POLLRDNORM;
	} 
	return 0;
}

static ssize_t fx_in_dev_write(struct file * file, const char * buf,
							 size_t count, loff_t *pos)
{
	struct sk_buff *skb;
	int instance = _find_file_instance(file);

	if(instance < 0)
		return -1;

	if(!fxroot[instance]->xmit && fxroot[instance]->dev && fxroot[instance]->dev->hard_start_xmit)
	{
		fxroot[instance]->xmit = fxroot[instance]->dev->hard_start_xmit;
		fxroot[instance]->dev->hard_start_xmit = fx_xmit;
	}

	if(!fxroot[instance]->xmit)
		return -1;

	skb = alloc_skb(count, GFP_ATOMIC);
	if(skb)
	{
		skb_put(skb, count);
		memcpy(skb->data, buf, count);
		skb->dev = fxroot[instance]->dev;
		fxroot[instance]->xmit(skb, fxroot[instance]->dev);
		return count;
	}
	return 0;
}

static ssize_t fx_in_dev_read(struct file * file, char * buf,
			      size_t count, loff_t *pos)
{
	int instance = _find_file_instance(file);
	struct sk_buff *skb;

	if(instance < 0)
		return -1;

	if((skb = skb_dequeue(&(fxroot[instance]->inbuf))) != NULL)
	{
		int len = MIN(count, skb->len);

		if(skb->mac.ethernet)
		{
			len = MIN(count, skb->len + 14);

			if(len > 14)
				memcpy(buf, skb->mac.ethernet, 14);
			else
                len = -1;
		}
		/* Copy the data into the destination buffer */
		if(len > 14)
			memcpy(&buf[14], skb->data, len);
		else
			len = -1;

		kfree_skb(skb);

		return len;
	}
	return 0;
}

static loff_t fx_in_dev_lseek(struct file * file, loff_t offset, int origin)
{
	return -ESPIPE;
}

static int fx_in_dev_open(struct inode * inode, struct file * file)
{
	MOD_INC_USE_COUNT;
	return 0;
}

static int fx_in_dev_release(struct inode * inode, struct file * file)
{
	MOD_DEC_USE_COUNT;
	return 0;
}

static int fx_in_dev_ioctl(struct inode *inode, struct file *file,
						unsigned int cmd, unsigned long arg)
{
	/* We don't have any FX specific ioctls at the moment */
	return 0;
}

static struct file_operations fx_in_dev_fops = {
	llseek:		fx_in_dev_lseek,
	read:		fx_in_dev_read,
	write:		fx_in_dev_write,
	poll:		fx_in_dev_poll,
	ioctl:		fx_in_dev_ioctl,
	open:		fx_in_dev_open,
	release:	fx_in_dev_release,
};

/* Dealing with the out (from kernel TCP/IP) device */
static unsigned int fx_out_dev_poll(struct file *file, poll_table * wait)
{
	int instance = _find_file_instance(file);

	if(instance > -1)
	{
		if(skb_queue_len(&(fxroot[instance]->outbuf)))
			return POLLIN | POLLRDNORM;

		poll_wait(file, &(fxroot[instance]->outwait), wait);

		if(skb_queue_len(&(fxroot[instance]->outbuf)))
			return POLLIN | POLLRDNORM;
	}
	return 0;
}

static ssize_t fx_out_dev_write(struct file * file, const char * buf,
								size_t count, loff_t *pos)
{
	struct sk_buff *skb;
	int instance = _find_file_instance(file);

	if(instance < 0 || count < 15)
		return -1;

	skb = alloc_skb(count - 14, GFP_ATOMIC);
	if(skb)
	{
		int result;
		short proto;

		skb->dev = fxroot[instance]->dev;
		skb_put(skb, count - 14);
		/* We get a MAC (ethernet) header but currently
		 * we don't use it.  So we just start copying
		 * at the start of the data.
		 */
		memcpy(skb->data, &buf[14], count - 14);
		skb->mac.raw = skb->data;
		proto = *((short *)&buf[12]);
		skb->protocol = proto;
		result = netif_rx(skb);
		if(fxroot[instance]->ppp)
		{
			fxroot[instance]->stats.rx_packets++;
			fxroot[instance]->stats.rx_bytes += count;
		}

		switch(result)
		{
		case NET_RX_DROP:
			printk(KERN_INFO "Packet dropped to interface fx%d while offering up.\n", instance);
			break;
		case NET_RX_CN_LOW:
			printk(KERN_INFO "Low congestion on interface fx%d while offering up.\n", instance);
			break;
		case NET_RX_CN_MOD:
			printk(KERN_INFO "Moderate congestion on interface fx%d while offering up.\n", instance);
			break;
		case NET_RX_CN_HIGH:
			printk(KERN_INFO "High congestion on interface fx%d while offering up.\n", instance);
			break;
		}
		return count;
	}
	return 0;
}

static ssize_t fx_out_dev_read(struct file * file, char * buf,
			       size_t count, loff_t *pos)
{
	int instance = _find_file_instance(file);
	struct sk_buff *skb;

	if(instance < 0)
		return -1;

	/* We don't have any fragments so we grab a new packet */
	if((skb = skb_dequeue(&(fxroot[instance]->outbuf))) != NULL)
	{
		int len = MIN(count, skb->len);

		/* Copy the data into the destination buffer */
		memcpy(buf, skb->data, len);

		kfree_skb(skb);

		return len;
	}
	return 0;
}

static loff_t fx_out_dev_lseek(struct file * file, loff_t offset, int origin)
{
	return -ESPIPE;
}

static int fx_out_dev_open(struct inode * inode, struct file * file)
{
	MOD_INC_USE_COUNT;
	return 0;
}

static int fx_out_dev_release(struct inode * inode, struct file * file)
{
	MOD_DEC_USE_COUNT;
	return 0;
}

static int fx_out_dev_ioctl(struct inode *inode, struct file *file,
							unsigned int cmd, unsigned long arg)
{
	/* We don't have any FX specific ioctls at the moment */
	return 0;
}

static struct file_operations fx_out_dev_fops = {
	llseek:		fx_out_dev_lseek,
	read:		fx_out_dev_read,
	write:		fx_out_dev_write,
	poll:		fx_out_dev_poll,
	ioctl:		fx_out_dev_ioctl,
	open:		fx_out_dev_open,
	release:	fx_out_dev_release,
};

/* Generic */
static int fx_generic_open(struct inode * inode, struct file * file)
{
	int len = 0;
	char *name;

	if(!file || !file->f_dentry)
		return -ENODEV;

	name = (char *)file->f_dentry->d_name.name;
	len = file->f_dentry->d_name.len;

	/* On a non-devfs system we have to set the file_operation
	 * structs ourselves since devfs_register() doesn't.
	 */
	if(len > 3 && strncmp("fxin", name, 4) == 0)
	{
		file->f_op = &fx_in_dev_fops;
		return fx_in_dev_open(inode, file);
	}
	else if(len > 4 && strncmp("fxout", name, 5) == 0)
	{
		file->f_op = &fx_out_dev_fops;
		return fx_out_dev_open(inode, file);
	}
	return -ENODEV;
}


static struct file_operations fx_generic_fops = {
	open: fx_generic_open,
};

/* The next five functions are for overloading subdrivers */
void fx_unregister_netdev(struct net_device *x)
{
	int instance = _find_instance(x);

	if(instance > -1)
	{
		del_timer(&fxroot[instance]->timer);
		devfs_unregister(fxroot[instance]->inhandle);
		devfs_unregister(fxroot[instance]->outhandle);
		kfree(fxroot[instance]);
		fxroot[instance] = NULL;
	}

	MOD_DEC_USE_COUNT;
	unregister_netdev(x);
}

void fx_netif_rx(struct sk_buff *skb)
{
	int instance = _find_instance(skb->dev);

	/* May need to free the skb here */
	if(instance < 0)
	{
		kfree_skb(skb);
		return;
	}

	/* There have been cases where the timer fails
	 * so we double check here.
	 */
	if(!fxroot[instance]->xmit)
		dev_mon(instance);

	if(EnablementCode[9] == '0' && !fxroot[instance]->fx_active && !is_dhcp(skb))
	{
		kfree_skb(skb);
		return;
	}
	
	if(fxroot[instance]->fx_active || fxroot[instance]->ppp)
	{
		skb_queue_tail(&(fxroot[instance]->inbuf), skb);
		wake_up_interruptible(&(fxroot[instance]->inwait));
	}
	else
		netif_rx(skb);
}

/* For oldstyle drivers */
struct net_device *fx_init_etherdev(struct net_device *x, int y)
{
	struct net_device *dev = kmalloc(sizeof(struct net_device), GFP_KERNEL);
	memset(dev, 0, sizeof(struct net_device));

	if(y)
	{
		dev->priv = kmalloc(y, GFP_KERNEL);
		memset(dev->priv, 0, y);
	}
	MOD_INC_USE_COUNT;
	ether_setup(dev);
	fx_startup(dev, 1, 0);
	return dev;
}

/* For new style drivers */
int fx_register_netdev(struct net_device *dev)
{
	return fx_startup(dev, 0, 0);
}

int fx_dev_alloc_name(struct net_device *x, char *name)
{
	return 0;
}

/* Dealing with the interface */
int __init fx_init(struct net_device *dev)
{
	int instance = _find_instance(dev);

	if(instance > -1)
	{
		int res = 0;

		if(fxroot[instance]->init)
			res = fxroot[instance]->init(dev);

		fxroot[instance]->xmit = dev->hard_start_xmit;
		dev->hard_start_xmit = fx_xmit;

		return res;
	}

	return 0;
}

static int fx_xmit(struct sk_buff *skb, struct net_device *dev)
{
	int instance = _find_instance(dev);

	/* May need to free the skb here */
	if(instance < 0)
	{
		kfree_skb(skb);
		return -1;
	}

	if(fxroot[instance]->fx_active || fxroot[instance]->ppp)
	{
		if(fxroot[instance]->ppp)
		{
			fxroot[instance]->stats.tx_packets++;
			fxroot[instance]->stats.tx_bytes += skb->len;
		}
		skb_queue_tail(&(fxroot[instance]->outbuf), skb);
		wake_up_interruptible(&(fxroot[instance]->outwait));
	}
	else if(fxroot[instance]->xmit)
		fxroot[instance]->xmit(skb, fxroot[instance]->dev);
	else
		kfree_skb(skb);
	return 0;
}

static struct net_device_stats *fx_get_stats(struct net_device *dev)
{
	int instance = _find_instance(dev);

	if(instance < 0)
		return NULL;
	return &fxroot[instance]->stats;
}

/* Monitor the net_device structure waiting for function initialization */
void dev_mon(unsigned long instance)
{
	struct net_device *dev = fxroot[instance]->dev;

	if(dev->hard_start_xmit)
	{
        fxroot[instance]->xmit = dev->hard_start_xmit;
		dev->hard_start_xmit = fx_xmit;

		del_timer(&fxroot[instance]->timer);
		fxroot[instance]->count = 0;
		return;
	}

	fxroot[instance]->count++;

	/* We only try to check for 60 seconds */
	if(fxroot[instance]->count == 60)
	{
		del_timer(&fxroot[instance]->timer);
		fxroot[instance]->count = 0;
		return;
	}

	mod_timer(&fxroot[instance]->timer, HZ);
}

int fx_startup(struct net_device *dev, int old, int ppp)
{
	char name[20];
	int count = 0, err = -1;

	/* Find first free instance */
	while(fxroot[count])
	{
		count++;
		if(count == FX_MAX_INSTANCE)
			return -1;
	}

	if(ppp)
		ppp_instance = count;

	/* Allocate a buffer for the current instance */
	fxroot[count] = kmalloc(sizeof(FxBuf), GFP_KERNEL);
	memset(fxroot[count], 0, sizeof(FxBuf));

	fxroot[count]->ppp = ppp;
	fxroot[count]->dev = dev;
	fxroot[count]->init = dev->init;

	init_waitqueue_head(&(fxroot[count]->outwait));
	skb_queue_head_init(&(fxroot[count]->outbuf));

	if(!ppp)
	{
		init_waitqueue_head(&(fxroot[count]->inwait));
		skb_queue_head_init(&(fxroot[count]->inbuf));

		sprintf(name, "fxin%d", count);

		fxroot[count]->inhandle = devfs_register(devfs_handle, name, DEVFS_FL_DEFAULT,
												 fx_major, (count*2),
												 S_IFCHR | S_IRUSR | S_IWUSR,
												 &fx_in_dev_fops, NULL);
	}

	sprintf(name, "fxout%d", count);

	fxroot[count]->outhandle = devfs_register(devfs_handle, name, DEVFS_FL_DEFAULT,
											  fx_major, (count*2)+1,
											  S_IFCHR | S_IRUSR | S_IWUSR,
											  &fx_out_dev_fops, NULL);

	/* Find a name for this unit */
	err=dev_alloc_name(dev,"fx%d");

	if(err<0)
		return err;

    if(!old && !ppp)
		dev->init = fx_init;

	if(ppp)
		dev->get_stats = fx_get_stats;

	/* Ok we create an ethernet device */
	if(register_netdev(dev) != 0)
		return -EIO;

	if(old)
	{
		init_timer(&fxroot[count]->timer);
		fxroot[count]->timer.function = dev_mon;
		fxroot[count]->timer.data = (unsigned long)count;
		fxroot[count]->timer.expires = HZ;
		fxroot[count]->count = 0;
		add_timer(&fxroot[count]->timer);
	}

	return 0;
}

int fx_init_module(void)
{
	printk(KERN_INFO "FX Network Filter Driver Copyright 2000-2002\n");

	/* Clear the instance list */
	memset(fxroot, 0, sizeof(FxBuf*) * FX_MAX_INSTANCE);
	memset(pleaseexit, 0, sizeof(int) * FX_MAX_INSTANCE);

	/* We also create entries in /dev */
	if ((fx_major = devfs_register_chrdev(FX_MAJOR, "fx", &fx_generic_fops)) < 0)
	{
		printk(KERN_ERR "fx: unable to get major %d (%d)\n", FX_MAJOR, fx_major);
		return -EIO;
	}
	else if(fx_major == 0)
	{
		/* In the case that devfs isn't installed we will
		 * get a return code of zero so set fx_major to the
		 * requested major.
		 */
		fx_major = FX_MAJOR;
	}

	/* We'll put them under the /dev/fx directory */
	devfs_handle = devfs_mk_dir (NULL, "fx", NULL);

	fx_ent = create_proc_entry("fx", S_IFREG | S_IRUGO | S_IWUGO,
				   &proc_root);

	if(fx_ent)
	{
		fx_ent->read_proc = fx_proc_read;
		fx_ent->write_proc = fx_proc_write;
	}

	printk(KERN_INFO "Initialization complete. FX Major is %u.\n", fx_major);

	return 0;
}

void fx_cleanup_module(void)
{
	int z;

	for(z=0;z<FX_MAX_INSTANCE;z++)
	{
		if(fxroot[z])
			kfree(fxroot[z]);
	}

	devfs_unregister(devfs_handle);
	devfs_unregister_chrdev(fx_major, "fx");
	remove_proc_entry("fx", &proc_root);

	printk(KERN_INFO "FX Network Filter Driver successfully unloaded.\n");
}

MODULE_AUTHOR("F/X Communications <info@fx.dk>");
MODULE_DESCRIPTION("F/X Communications Network Filter");
#ifdef MODULE_LICENSE
MODULE_LICENSE("GPL");
#endif

/* 8390.c gets statically linked into the kernel in Linux 2.4.x, and
 * any NE2000 based cards that use it will bypass our frame redirection,
 * so we need to export these functions ourself but to avoid conflicts
 * we must rename the functions, which is what the #defines below are for.
 */
#ifdef MODULE
#undef MODULE
#endif
#undef ei_open
#define ei_open fx_ei_open
#undef ei_close
#define ei_close fx_ei_close
#undef ei_interrupt
#define ei_interrupt fx_ei_interrupt
#undef ei_tx_timeout
#define ei_tx_timeout fx_ei_tx_timeout
#undef ethdev_init
#define ethdev_init fx_ethdev_init
#undef NS8390_init
#define NS8390_init fx_NS8390_init
#undef netif_rx
#define netif_rx fx_netif_rx
#undef MODULE_LICENSE
#define MODULE_LICENSE(a)
#include "8390.c"

EXPORT_SYMBOL(fx_netif_rx);
EXPORT_SYMBOL(fx_register_netdev);
EXPORT_SYMBOL(fx_unregister_netdev);
EXPORT_SYMBOL(fx_init_etherdev);
EXPORT_SYMBOL(fx_dev_alloc_name);

module_init(fx_init_module);
module_exit(fx_cleanup_module);
